@@ -62,6 +62,8 @@ namespace rocrtst {
   } \
 }
 
+size_t pool_size_limit = 0;
+
 static hsa_status_t FindAgent(hsa_agent_t agent, void* data,
                                                 hsa_device_type_t dev_type) {
   assert(data != nullptr);
@@ -405,6 +407,17 @@ hsa_status_t AcquirePoolInfo(hsa_amd_memory_pool_t pool,
   const size_t max_pool_size = 2*1024*1024*1024UL;
   pool_i->size = std::min(pool_i->size, max_pool_size);
 #endif
+  pool_size_limit = 0;
+  char *pool_size_limit_str = getenv("ROCRTST_LIMIT_POOL_SIZE");
+  if (pool_size_limit_str) {
+    char *end;
+    pool_size_limit = strtoul(pool_size_limit_str, &end, 10);
+    if (pool_size_limit > pool_i->size) {
+      std::cout << "Warning: Pool size override > than reported size (override:"
+        << pool_size_limit << " reported:" << pool_i->size << ")" << std::endl;
+    }
+    pool_i->size = pool_size_limit;
+  }
 
   err = hsa_amd_memory_pool_get_info(pool,
              HSA_AMD_MEMORY_POOL_INFO_RUNTIME_ALLOC_ALLOWED,
