@@ -332,10 +332,24 @@ hsa_status_t XdnaDriver::AllocQueueGWS(HSA_QUEUEID queue_id, uint32_t num_gws,
   return HSA_STATUS_ERROR_INVALID_QUEUE;
 }
 
-hsa_status_t XdnaDriver::ExportDMABuf(void *mem, size_t size, int *dmabuf_fd,
-                                      size_t *offset) {
-  // Not implemented yet.
-  return HSA_STATUS_ERROR;
+hsa_status_t XdnaDriver::ExportDMABuf(void* mem, size_t size, int* dmabuf_fd, size_t* offset) {
+  auto bo_handle = FindBOHandle(mem);
+  if (!bo_handle.IsValid()) {
+    return HSA_STATUS_ERROR_INVALID_ALLOCATION;
+  }
+
+  drm_prime_handle export_params = {};
+  export_params.handle = bo_handle.handle;
+  export_params.flags = DRM_RDWR;
+  export_params.fd = -1;
+  if (ioctl(fd_, DRM_IOCTL_PRIME_HANDLE_TO_FD, &export_params) < 0) {
+    return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
+  }
+
+  *dmabuf_fd = export_params.fd;
+  *offset = reinterpret_cast<uintptr_t>(mem) - reinterpret_cast<uintptr_t>(bo_handle.vaddr);
+
+  return HSA_STATUS_SUCCESS;
 }
 
 hsa_status_t XdnaDriver::ImportDMABuf(int dmabuf_fd, core::Agent &agent,
@@ -877,5 +891,44 @@ hsa_status_t XdnaDriver::SetTrapHandler(uint32_t node_id, const void* base, uint
   return HSA_STATUS_ERROR;
 }
 
+hsa_status_t XdnaDriver::AllocateScratchMemory(uint32_t node_id, uint64_t size, void** mem) const {
+  return HSA_STATUS_ERROR;
+}
+
+hsa_status_t XdnaDriver::GetDeviceHandle(uint32_t node_id, void** device_handle) const {
+  return HSA_STATUS_ERROR;
+}
+
+hsa_status_t XdnaDriver::GetClockCounters(uint32_t node_id, HsaClockCounters* clock_counter) const {
+  return HSA_STATUS_ERROR;
+}
+
+
+hsa_status_t XdnaDriver::GetTileConfig(uint32_t node_id, HsaGpuTileConfig* config) const {
+  return HSA_STATUS_ERROR;
+}
+
+hsa_status_t XdnaDriver::GetWallclockFrequency(uint32_t node_id, uint64_t* frequency) const {
+  return HSA_STATUS_ERROR;
+}
+
+hsa_status_t XdnaDriver::AvailableMemory(uint32_t node_id, uint64_t* available_size) const {
+  return HSA_STATUS_ERROR;
+}
+
+hsa_status_t XdnaDriver::RegisterMemory(void* ptr, uint64_t size, HsaMemFlags mem_flags) const {
+  return HSA_STATUS_ERROR;
+}
+
+hsa_status_t XdnaDriver::DeregisterMemory(void* ptr) const { return HSA_STATUS_ERROR; }
+
+hsa_status_t XdnaDriver::MakeMemoryResident(const void* mem, size_t size, uint64_t* alternate_va,
+                                            const HsaMemMapFlags* mem_flags, uint32_t num_nodes,
+                                            const uint32_t* nodes) const {
+  return HSA_STATUS_ERROR;
+}
+
+hsa_status_t XdnaDriver::MakeMemoryUnresident(const void* mem) const { return HSA_STATUS_ERROR; }
+
 } // namespace AMD
 } // namespace rocr
