@@ -3,7 +3,7 @@
 // The University of Illinois/NCSA
 // Open Source License (NCSA)
 //
-// Copyright (c) 2014-2024, Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2014-2025, Advanced Micro Devices, Inc. All rights reserved.
 //
 // Developed by:
 //
@@ -63,13 +63,13 @@
 #include "core/inc/hsa_ext_amd_impl.h"
 
 #include "core/inc/agent.h"
-#include "core/inc/amd_kfd_driver.h"
-#include "core/inc/amd_xdna_driver.h"
+#include "core/inc/driver.h"
 #include "core/inc/exceptions.h"
 #include "core/inc/interrupt_signal.h"
 #include "core/inc/memory_region.h"
 #include "core/inc/signal.h"
 #include "core/inc/svm_profiler.h"
+#include "core/inc/thunk_loader.h"
 #include "core/util/flag.h"
 #include "core/util/locks.h"
 #include "core/util/os.h"
@@ -91,6 +91,7 @@
 #define HSA_ARGUMENT_ALIGN_BYTES 16
 #define HSA_QUEUE_ALIGN_BYTES 64
 #define HSA_PACKET_ALIGN_BYTES 64
+#define HSA_MAX_DEP_SIGNALS 5
 
 //Avoids include
 namespace rocr {
@@ -159,7 +160,7 @@ class Runtime {
 
   /// @brief Insert agent into the driver list.
   /// @param [in] driver Unique pointer to the driver object.
-  void RegisterDriver(std::unique_ptr<Driver> &driver);
+  void RegisterDriver(std::unique_ptr<Driver> driver);
 
   /// @brief Delete all agent objects from ::agents_.
   void DestroyAgents();
@@ -284,6 +285,16 @@ class Runtime {
   hsa_status_t CopyMemoryStatus(core::Agent* dst_agent, core::Agent* src_agent,
                                 uint32_t *engine_ids_mask);
 
+  /// @brief Get preferred SDMA engine for the copy direction
+  ///
+  /// @param [in] dst_agent Destination agent.
+  /// @param [in] src_agent Source agent.
+  /// @param [out] recommended_ids_mask Mask of recommended_ids.
+  ///
+  /// @retval HSA_STATUS_SUCCESS For mask returned
+  hsa_status_t GetPreferredEngine(core::Agent* dst_agent, core::Agent* src_agent,
+                                  uint32_t* recommended_ids_mask);
+
   /// @brief Fill the first @p count of uint32_t in ptr with value.
   ///
   /// @param [in] ptr Memory address to be filled.
@@ -365,7 +376,8 @@ class Runtime {
   hsa_status_t SvmPrefetch(void* ptr, size_t size, hsa_agent_t agent, uint32_t num_dep_signals,
                            const hsa_signal_t* dep_signals, hsa_signal_t completion_signal);
 
-  hsa_status_t DmaBufExport(const void* ptr, size_t size, int* dmabuf, uint64_t* offset);
+  hsa_status_t DmaBufExport(const void* ptr, size_t size, int* dmabuf,
+                                            uint64_t* offset, uint64_t flags);
 
   hsa_status_t DmaBufClose(int dmabuf);
 
@@ -443,6 +455,9 @@ class Runtime {
   }
 
   const Flag& flag() const { return flag_; }
+  Flag& flag() { return flag_; }
+
+  const ThunkLoader* thunkLoader() const { return thunkLoader_; }
 
   ExtensionEntryPoints extensions_;
 
@@ -465,6 +480,10 @@ class Runtime {
     if (version.KernelInterfaceMajorVersion == 1 &&
       version.KernelInterfaceMinorVersion >= 14)
       kfd_version.supports_event_age = true;
+
+    if (thunkLoader()->IsDXG()) {
+      kfd_version.supports_event_age = false;
+    }
   }
 
   void KfdVersion(bool exception_debugging, bool core_dump) {
@@ -494,6 +513,16 @@ class Runtime {
     return **driver;
   }
 
+  std::vector<std::unique_ptr<Driver>>& AgentDrivers() { return agent_drivers_; }
+
+  static bool IsGPUDriver(DriverType driver_type) {
+    return driver_type == core::DriverType::KFD
+#ifdef HSAKMT_VIRTIO_ENABLED
+        || driver_type == core::DriverType::KFD_VIRTIO
+#endif
+        ;
+  }
+
  protected:
   static void AsyncEventsLoop(void*);
   static void AsyncIPCSockServerConnLoop(void*);
@@ -512,7 +541,8 @@ class Runtime {
           size(size_arg),
           size_requested(size_requested),
           alloc_flags(alloc_flags),
-          user_ptr(nullptr) {}
+          user_ptr(nullptr),
+          ldrm_bo(NULL) {}
 
     struct notifier_t {
       void* ptr;
@@ -756,6 +786,8 @@ class Runtime {
   // Track environment variables.
   Flag flag_;
 
+  ThunkLoader* thunkLoader_;
+
   // Pools memory for SharedSignal (Signal ABI blocks)
   SharedSignalPool_t SharedSignalPool;
 
@@ -774,7 +806,8 @@ class Runtime {
 
  private:
   void CheckVirtualMemApiSupport();
-  int GetAmdgpuDeviceArgs(Agent* agent, amdgpu_bo_handle bo, int* drm_fd, uint64_t* cpu_addr);
+  int GetAmdgpuDeviceArgs(Agent *agent, ShareableHandle handle, int *drm_fd,
+                          uint64_t *cpu_addr);
 
   bool virtual_mem_api_supported_;
   bool xnack_enabled_;
@@ -782,16 +815,18 @@ class Runtime {
   typedef void* ThunkHandle;
 
   struct AddressHandle {
-    AddressHandle() : size(0), use_count(0) {}
-    AddressHandle(size_t size) : size(size), use_count(0) {}
+    AddressHandle() : os_addr(nullptr), size(0), use_count(0), registered(false) {}
+    AddressHandle(void* addr, size_t _size, bool _registered) : os_addr(addr), size(_size), use_count(0), registered(_registered) {}
 
+    // Address returned by OS. May be different from user address when adjusted for alignment
+    void *os_addr;
     size_t size;
     int use_count;
+    bool registered;
   };
   std::map<const void*, AddressHandle> reserved_address_map_;  // Indexed by VA
 
   struct MemoryHandle {
-    MemoryHandle() : region(NULL), size(0), ref_count(0), thunk_handle(NULL), alloc_flag(0) {}
     MemoryHandle(const MemoryRegion* region, size_t size, uint64_t flags_unused,
                  ThunkHandle thunk_handle, MemoryRegion::AllocateFlags alloc_flag)
         : region(region),
@@ -801,27 +836,29 @@ class Runtime {
           thunk_handle(thunk_handle),
           alloc_flag(alloc_flag) {}
 
-    static __forceinline hsa_amd_vmem_alloc_handle_t Convert(void* handle) {
+    static __forceinline hsa_amd_vmem_alloc_handle_t Convert(ThunkHandle handle) {
       hsa_amd_vmem_alloc_handle_t ret_handle = {
           static_cast<uint64_t>(reinterpret_cast<uintptr_t>(handle))};
       return ret_handle;
     }
 
+    static __forceinline ThunkHandle Convert(hsa_amd_vmem_alloc_handle_t handle) {
+      return reinterpret_cast<void*>(handle.handle);
+    }
+
     __forceinline core::Agent* agentOwner() const { return region->owner(); }
 
     const MemoryRegion* region;
     size_t size;
     int ref_count;
     int use_count;
-    ThunkHandle thunk_handle;  // handle returned by hsaKmtAllocMemory(NoAddress = 1)
+    ThunkHandle thunk_handle;  // handle returned by Driver::Allocate(NoAddress = 1)
     MemoryRegion::AllocateFlags alloc_flag;
   };
   std::map<ThunkHandle, MemoryHandle> memory_handle_map_;
 
   struct MappedHandle;
   struct MappedHandleAllowedAgent {
-    MappedHandleAllowedAgent()
-        : va(NULL), permissions(HSA_ACCESS_PERMISSION_NONE), mappedHandle(NULL), ldrm_bo(0) {}
     MappedHandleAllowedAgent(MappedHandle* _mappedHandle, Agent* targetAgent, void* va, size_t size,
                              hsa_access_permission_t perms);
     ~MappedHandleAllowedAgent();
@@ -834,46 +871,34 @@ class Runtime {
     Agent* targetAgent;
     hsa_access_permission_t permissions;
     MappedHandle* mappedHandle;
-    amdgpu_bo_handle ldrm_bo;
+    ShareableHandle shareable_handle;
   };
 
   struct MappedHandle {
-    MappedHandle()
-        : mem_handle(NULL),
-          address_handle(NULL),
-          offset(0),
-          mmap_offset(0),
-          size(0),
-          drm_fd(-1),
-          drm_cpu_addr(NULL),
-          ldrm_bo(0) {}
-
-    MappedHandle(MemoryHandle* mem_handle, AddressHandle* address_handle, uint64_t offset,
-                 size_t size, int drm_fd, void* drm_cpu_addr, hsa_access_permission_t perm,
-                 amdgpu_bo_handle bo)
-        : mem_handle(mem_handle),
-          address_handle(address_handle),
-          offset(offset),
-          mmap_offset(0),
-          size(size),
-          drm_fd(drm_fd),
-          drm_cpu_addr(drm_cpu_addr),
-          ldrm_bo(bo) {}
+    MappedHandle(MemoryHandle *mem_handle, AddressHandle *address_handle,
+                 uint64_t offset, size_t size, int drm_fd, void *drm_cpu_addr,
+                 hsa_access_permission_t perm, ShareableHandle shareable_handle)
+        : mem_handle(mem_handle), address_handle(address_handle),
+          offset(offset), size(size), drm_fd(drm_fd),
+          drm_cpu_addr(drm_cpu_addr), shareable_handle(shareable_handle) {}
 
     __forceinline core::Agent* agentOwner() const { return mem_handle->region->owner(); }
 
     MemoryHandle* mem_handle;
     AddressHandle* address_handle;
     uint64_t offset;
-    uint64_t mmap_offset;
     size_t size;
     int drm_fd;
     void* drm_cpu_addr;  // CPU Buffer address
-    amdgpu_bo_handle ldrm_bo;
+    ShareableHandle shareable_handle;
     std::map<Agent*, MappedHandleAllowedAgent> allowed_agents;
   };
   std::map<const void*, MappedHandle> mapped_handle_map_;  // Indexed by VA
 
+  AddressHandle* VMemoryFindReservedAddressHandle(const void* va);
+  hsa_status_t VMemoryPtrInfo(const void* ptr, hsa_amd_pointer_info_t* info, void* (*alloc)(size_t),
+                              uint32_t* num_agents_accessible, hsa_agent_t** accessible);
+
   hsa_status_t VMemoryMapAllowAccess(const void *va,
                                      hsa_access_permission_t perm,
                                      const hsa_agent_t *agents,
@@ -883,11 +908,6 @@ class Runtime {
                             const hsa_amd_memory_access_desc_t *desc,
                             const size_t desc_cnt);
 
-  // Frees runtime memory when the runtime library is unloaded if safe to do so.
-  // Failure to release the runtime indicates an incorrect application but is
-  // common (example: calls library routines at process exit).
-  friend class RuntimeCleanup;
-
   void InitIPCDmaBufSupport();
   bool ipc_dmabuf_supported_;
   int  IPCClientImport(uint32_t conn_handle, uint64_t dmabuf_fd_handle,
