@@ -43,7 +43,6 @@
 #include "core/inc/amd_memory_region.h"
 
 #include <algorithm>
-#include <set>
 
 #include "core/inc/runtime.h"
 #include "core/inc/amd_cpu_agent.h"
@@ -57,34 +56,7 @@ namespace AMD {
 
 // Tracks aggregate size of system memory available on platform
 size_t MemoryRegion::max_sysmem_alloc_size_ = 0;
-
-bool MemoryRegion::RegisterMemory(void* ptr, size_t size, const HsaMemFlags& MemFlags) {
-  assert(ptr != NULL);
-  assert(size != 0);
-
-  const HSAKMT_STATUS status = hsaKmtRegisterMemoryWithFlags(ptr, size, MemFlags);
-  return (status == HSAKMT_STATUS_SUCCESS);
-}
-
-void MemoryRegion::DeregisterMemory(void* ptr) { hsaKmtDeregisterMemory(ptr); }
-
-bool MemoryRegion::MakeKfdMemoryResident(size_t num_node, const uint32_t* nodes, const void* ptr,
-                                         size_t size, uint64_t* alternate_va,
-                                         HsaMemMapFlags map_flag) {
-  assert(num_node > 0);
-  assert(nodes != NULL);
-
-  *alternate_va = 0;
-  const HSAKMT_STATUS status = hsaKmtMapMemoryToGPUNodes(
-      const_cast<void*>(ptr), size, alternate_va, map_flag, num_node, const_cast<uint32_t*>(nodes));
-
-  return (status == HSAKMT_STATUS_SUCCESS);
-}
-
-bool MemoryRegion::MakeKfdMemoryUnresident(const void* ptr) {
-  const HSAKMT_STATUS status = hsaKmtUnmapMemoryToGPU(const_cast<void*>(ptr));
-  return (status == HSAKMT_STATUS_SUCCESS);
-}
+const size_t MemoryRegion::kPageSize_ = sysconf(_SC_PAGESIZE);
 
 MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
                            bool extended_scope_fine_grain, bool user_visible, core::Agent* owner,
@@ -124,7 +96,7 @@ MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
     virtual_size_ = kGpuVmSize;
 
   } else if (IsSystem()) {
-    mem_flag_.ui32.PageSize = MemoryRegion::kPageSize();
+    mem_flag_.ui32.PageSize = GetPageSize();
     mem_flag_.ui32.NoSubstitute = 0;
     mem_flag_.ui32.HostAccess = 1;
     mem_flag_.ui32.CachePolicy = HSA_CACHING_CACHED;
@@ -137,7 +109,7 @@ MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
 
 
   // Adjust allocatable size per page align
-  max_single_alloc_size_ = AlignDown(static_cast<size_t>(GetPhysicalSize()), kPageSize());
+  max_single_alloc_size_ = AlignDown(static_cast<size_t>(GetPhysicalSize()), GetPageSize());
 
   // Keep track of total system memory available
   // @note: System memory is surfaced as both coarse
@@ -149,7 +121,7 @@ MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
   }
 
   assert(GetVirtualSize() != 0);
-  assert(IsMultipleOf(max_single_alloc_size_, kPageSize()));
+  assert(IsMultipleOf(max_single_alloc_size_, GetPageSize()));
 }
 
 MemoryRegion::~MemoryRegion() {}
@@ -176,10 +148,10 @@ hsa_status_t MemoryRegion::AllocateImpl(size_t& size, AllocateFlags alloc_flags,
     return HSA_STATUS_ERROR_INVALID_ALLOCATION;
   }
 
-  size = AlignUp(size, kPageSize());
+  size = AlignUp(size, GetPageSize());
 
-  return core::Runtime::runtime_singleton_->AgentDriver(owner()->driver_type)
-      .AllocateMemory(*this, alloc_flags, address, size, agent_node_id);
+  return owner()->driver().AllocateMemory(*this, alloc_flags, address, size,
+                                          agent_node_id);
 }
 
 hsa_status_t MemoryRegion::Free(void* address, size_t size) const {
@@ -190,8 +162,7 @@ hsa_status_t MemoryRegion::Free(void* address, size_t size) const {
 hsa_status_t MemoryRegion::FreeImpl(void* address, size_t size) const {
   if (fragment_allocator_.free(address)) return HSA_STATUS_SUCCESS;
 
-  return core::Runtime::runtime_singleton_->AgentDriver(owner()->driver_type)
-      .FreeMemory(address, size);
+  return owner()->driver().FreeMemory(address, size);
 }
 
 // TODO:  Look into a better name and/or making this process transparent to exporting.
@@ -278,7 +249,7 @@ hsa_status_t MemoryRegion::GetInfo(hsa_region_info_t attribute,
         case HSA_HEAPTYPE_DEVICE_SVM:
         case HSA_HEAPTYPE_FRAME_BUFFER_PRIVATE:
         case HSA_HEAPTYPE_FRAME_BUFFER_PUBLIC:
-          *((size_t*)value) = kPageSize();
+          *((size_t*)value) = GetPageSize();
           break;
         default:
           *((size_t*)value) = 0;
@@ -291,7 +262,7 @@ hsa_status_t MemoryRegion::GetInfo(hsa_region_info_t attribute,
         case HSA_HEAPTYPE_DEVICE_SVM:
         case HSA_HEAPTYPE_FRAME_BUFFER_PRIVATE:
         case HSA_HEAPTYPE_FRAME_BUFFER_PUBLIC:
-          *((size_t*)value) = kPageSize();
+          *((size_t*)value) = GetPageSize();
           break;
         default:
           *((size_t*)value) = 0;
@@ -336,19 +307,7 @@ hsa_status_t MemoryRegion::GetPoolInfo(hsa_amd_memory_pool_info_t attribute,
       *((bool*)value) = IsSystem() ? true : false;
       break;
     case HSA_AMD_MEMORY_POOL_INFO_ALLOC_MAX_SIZE:
-      switch (mem_props_.HeapType) {
-        case HSA_HEAPTYPE_FRAME_BUFFER_PRIVATE:
-        case HSA_HEAPTYPE_FRAME_BUFFER_PUBLIC:
-        case HSA_HEAPTYPE_GPU_SCRATCH:
-          return GetInfo(HSA_REGION_INFO_ALLOC_MAX_SIZE, value);
-        case HSA_HEAPTYPE_SYSTEM:
-          // Aggregate size available for allocation
-          *((size_t*)value) = max_sysmem_alloc_size_;
-          break;
-        default:
-          *((size_t*)value) = 0;
-      }
-      break;
+      return GetInfo(HSA_REGION_INFO_ALLOC_MAX_SIZE, value);
     case HSA_AMD_MEMORY_POOL_INFO_LOCATION:
       if (IsLocalMemory())
         *((hsa_amd_memory_pool_location_t*)value) = HSA_AMD_MEMORY_POOL_LOCATION_GPU;
@@ -360,12 +319,12 @@ hsa_status_t MemoryRegion::GetPoolInfo(hsa_amd_memory_pool_info_t attribute,
     case HSA_AMD_MEMORY_POOL_INFO_RUNTIME_ALLOC_REC_GRANULE:
       switch (mem_props_.HeapType) {
         case HSA_HEAPTYPE_SYSTEM:
-          *((size_t*)value) = kPageSize();
+          *((size_t*)value) = GetPageSize();
           break;
         case HSA_HEAPTYPE_FRAME_BUFFER_PRIVATE:
         case HSA_HEAPTYPE_FRAME_BUFFER_PUBLIC:
           *((size_t*)value) = core::Runtime::runtime_singleton_->flag().disable_fragment_alloc()
-              ? kPageSize()
+              ? GetPageSize()
               : fragment_allocator_.default_block_size();
           break;
         default:
@@ -476,11 +435,11 @@ hsa_status_t MemoryRegion::AllowAccess(uint32_t num_agents,
 
   // Adjust for fragments.  Make accessibility sticky for fragments since this will satisfy the
   // union of accessible agents between the fragments in the block.
-  hsa_amd_pointer_info_t info;
+  hsa_amd_pointer_info_t info = {};
   uint32_t agent_count = 0;
   hsa_agent_t* accessible = nullptr;
   MAKE_SCOPE_GUARD([&]() { free(accessible); });
-  core::Runtime::PtrInfoBlockData blockInfo;
+  core::Runtime::PtrInfoBlockData blockInfo = {};
   std::vector<uint64_t> union_agents;
   info.size = sizeof(info);
 
@@ -507,7 +466,6 @@ hsa_status_t MemoryRegion::AllowAccess(uint32_t num_agents,
 
   bool cpu_in_list = false;
 
-  std::set<GpuAgentInt*> whitelist_gpus;
   std::vector<uint32_t> whitelist_nodes;
   for (uint32_t i = 0; i < num_agents; ++i) {
     core::Agent* agent = core::Agent::Convert(agents[i]);
@@ -515,11 +473,16 @@ hsa_status_t MemoryRegion::AllowAccess(uint32_t num_agents,
       return HSA_STATUS_ERROR_INVALID_AGENT;
     }
 
-    if (agent->device_type() == core::Agent::kAmdGpuDevice) {
+    switch (agent->device_type()) {
+    case core::Agent::kAmdGpuDevice:
       whitelist_nodes.push_back(agent->node_id());
-      whitelist_gpus.insert(reinterpret_cast<GpuAgentInt*>(agent));
-    } else {
+      break;
+    case core::Agent::kAmdCpuDevice:
       cpu_in_list = true;
+      break;
+    case core::Agent::kAmdAieDevice:
+    default:
+      return HSA_STATUS_ERROR_INVALID_AGENT;
     }
   }
 
@@ -527,7 +490,7 @@ hsa_status_t MemoryRegion::AllowAccess(uint32_t num_agents,
     assert(cpu_in_list);
     // This is a system region and only CPU agents in the whitelist.
     // Remove old mappings.
-    AMD::MemoryRegion::MakeKfdMemoryUnresident(ptr);
+    owner()->driver().MakeMemoryUnresident(ptr);
     return HSA_STATUS_SUCCESS;
   }
 
@@ -537,7 +500,6 @@ hsa_status_t MemoryRegion::AllowAccess(uint32_t num_agents,
       std::find(whitelist_nodes.begin(), whitelist_nodes.end(), owner()->node_id()) ==
           whitelist_nodes.end()) {
     whitelist_nodes.push_back(owner()->node_id());
-    whitelist_gpus.insert(reinterpret_cast<GpuAgentInt*>(owner()));
   }
 
   HsaMemMapFlags map_flag = map_flag_;
@@ -548,10 +510,10 @@ hsa_status_t MemoryRegion::AllowAccess(uint32_t num_agents,
     ScopedAcquire<KernelSharedMutex::Shared> lock(
         core::Runtime::runtime_singleton_->memory_lock_.shared());
     uint64_t alternate_va = 0;
-    if (!AMD::MemoryRegion::MakeKfdMemoryResident(
-      whitelist_nodes.size(), &whitelist_nodes[0], ptr,
-      size, &alternate_va, map_flag)) {
-        return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
+    if (owner()->driver().MakeMemoryResident(ptr, size, &alternate_va, &map_flag,
+                                             whitelist_nodes.size(),
+                                             whitelist_nodes.data()) != HSA_STATUS_SUCCESS) {
+      return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
     }
   }
 
@@ -583,15 +545,10 @@ hsa_status_t MemoryRegion::Lock(uint32_t num_agents, const hsa_agent_t* agents,
     return HSA_STATUS_SUCCESS;
   }
 
-  std::set<core::Agent*> whitelist_gpus;
   std::vector<HSAuint32> whitelist_nodes;
   if (num_agents == 0 || agents == NULL) {
     // Map to all GPU agents.
     whitelist_nodes = core::Runtime::runtime_singleton_->gpu_ids();
-
-    whitelist_gpus.insert(
-        core::Runtime::runtime_singleton_->gpu_agents().begin(),
-        core::Runtime::runtime_singleton_->gpu_agents().end());
   } else {
     for (uint32_t i = 0; i < num_agents; ++i) {
       core::Agent* agent = core::Agent::Convert(agents[i]);
@@ -599,9 +556,16 @@ hsa_status_t MemoryRegion::Lock(uint32_t num_agents, const hsa_agent_t* agents,
         return HSA_STATUS_ERROR_INVALID_AGENT;
       }
 
-      if (agent->device_type() == core::Agent::kAmdGpuDevice) {
+      switch (agent->device_type()) {
+      case core::Agent::kAmdGpuDevice:
         whitelist_nodes.push_back(agent->node_id());
-        whitelist_gpus.insert(agent);
+        break;
+      case core::Agent::kAmdCpuDevice:
+        // Do nothing.
+        break;
+      case core::Agent::kAmdAieDevice:
+      default:
+        return HSA_STATUS_ERROR_INVALID_AGENT;
       }
     }
   }
@@ -614,10 +578,12 @@ hsa_status_t MemoryRegion::Lock(uint32_t num_agents, const hsa_agent_t* agents,
   }
 
   // Call kernel driver to register and pin the memory.
-  if (RegisterMemory(host_ptr, size, mem_flag_)) {
+  if (owner()->driver().RegisterMemory(host_ptr, size, const_cast<HsaMemFlags&>(mem_flag_)) ==
+      HSA_STATUS_SUCCESS) {
     uint64_t alternate_va = 0;
-    if (MakeKfdMemoryResident(whitelist_nodes.size(), &whitelist_nodes[0],
-                              host_ptr, size, &alternate_va, map_flag_)) {
+    if (owner()->driver().MakeMemoryResident(host_ptr, size, &alternate_va, &map_flag_,
+                                             whitelist_nodes.size(),
+                                             whitelist_nodes.data()) == HSA_STATUS_SUCCESS) {
       if (alternate_va != 0) {
         *agent_ptr = reinterpret_cast<void*>(alternate_va);
       } else {
@@ -626,7 +592,7 @@ hsa_status_t MemoryRegion::Lock(uint32_t num_agents, const hsa_agent_t* agents,
 
       return HSA_STATUS_SUCCESS;
     }
-    AMD::MemoryRegion::DeregisterMemory(host_ptr);
+    owner()->driver().DeregisterMemory(host_ptr);
     return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
   }
 
@@ -642,8 +608,12 @@ hsa_status_t MemoryRegion::Unlock(void* host_ptr) const {
     return HSA_STATUS_SUCCESS;
   }
 
-  MakeKfdMemoryUnresident(host_ptr);
-  DeregisterMemory(host_ptr);
+  if (owner()->driver().MakeMemoryUnresident(host_ptr) != HSA_STATUS_SUCCESS) {
+    assert(false && "Failed to unmap host pointer");
+  }
+  if (owner()->driver().DeregisterMemory(host_ptr) != HSA_STATUS_SUCCESS) {
+    assert(false && "Failed to deregister host pointer");
+  }
 
   return HSA_STATUS_SUCCESS;
 }
