@@ -151,6 +151,16 @@ static __forceinline unsigned long long int strtoull(const char* str,
       rocr::log_printf(__FILENAME__, __LINE__, format, ##__VA_ARGS__);                             \
   } while (false);
 
+#define LogSignalDuration(flag, signal, msg)                                                       \
+  do {                                                                                             \
+    if (hsa_flag_isset64(log_flags, flag)) {                                                       \
+      amd_signal_t* amd_signal = reinterpret_cast<amd_signal_t*>(signal.handle);                   \
+      rocr::log_printf(__FILENAME__, __LINE__,                                                     \
+        "%s Signal = (0x%lx), ticks start/end = %lu / %lu, Ticks elapsed = %lu", msg, signal,      \
+        amd_signal->start_ts, amd_signal->end_ts, amd_signal->end_ts - amd_signal->start_ts);      \
+    }                                                                                              \
+  } while (false);
+
 // A macro to remove unused variable warnings
 #define UNUSED(x) (void)(x)
 
@@ -169,7 +179,7 @@ class ScopeGuard {
 
   ScopeGuard(ScopeGuard& rhs) { *this = rhs; }
 
-  __forceinline ~ScopeGuard() {
+  __forceinline ~ScopeGuard() noexcept(false) {
     if (!dismiss_) release_();
   }
   __forceinline ScopeGuard& operator=(ScopeGuard& rhs) {
@@ -359,7 +369,11 @@ inline void FlushCpuCache(const void* base, size_t offset, size_t len) {
   static long cacheline_size = 0;
 
   if (!cacheline_size) {
-    long sz = sysconf(_SC_LEVEL1_DCACHE_LINESIZE);
+#ifdef _SC_LEVEL1_DCACHE_LINESIZE
+		long sz = sysconf(_SC_LEVEL1_DCACHE_LINESIZE);
+#else
+		long sz = 0;
+#endif
     if (sz <= 0) return;
     cacheline_size = sz;
   }
